//
// Created by vlad on 23.03.23.
//

#include "render.h"
#include "graphics/MeshLoader.h"
#include "graphics/Shader.h"

#include <glm/ext/matrix_transform.hpp>

static Mesh* mesh_xyz;
static Mesh* mesh_snowman;

static Shader* shader;


static void drawSnowman() {
    auto model_scale = glm::scale(glm::mat4(1.0f), glm::vec3(0.5f, 0.5f, 0.5f));

    static float angle = 0.0f;

    auto model_translate = glm::translate(glm::mat4(1.0f), glm::vec3(3, 2, 5));
    auto model_rotate = glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0, 1, 0));
    angle += 0.0002f;

    shader->uniformMatrix("model", model_scale * model_translate * model_rotate);
    mesh_snowman->draw();
}


void renderScene(Camera& cam) {
    auto projview = cam.getProjection() * cam.getView();
    shader->use();

//    shader->uniformMatrix("pr", cam.getProjection() * cam.getView());
    shader->uniformMatrix("projview", projview);
    shader->uniformMatrix("model", glm::mat4(1.0f));
    mesh_xyz->draw();

    drawSnowman();
}

int loadResources() {
    mesh_xyz = MeshLoader::loadMesh("res/xyz");
    if (mesh_xyz == nullptr) {
        return -1;
    }

    mesh_snowman = MeshLoader::loadMesh("res/snowman");
    if (mesh_snowman == nullptr) {
        return -1;
    }

    shader = Shader::loadShader("res/main.vsh", "res/main.fsh");
    if (shader == nullptr) {
        return -1;
    }

    return 0;
}

void unloadResources() {
    delete mesh_xyz;
    delete mesh_snowman;
    delete shader;
}